/*
DC_BrowseQuery() is only a sample class and not documented.
This class is used to provide a popup window of a user-friendly
dialog for creating filters for the browse.
*/

#INCLUDE "dcdialog.CH"
#INCLUDE "appevent.ch"

#Pragma Library("dclip1.lib")

#define COLOR_DARKGREEN  GraMakeRGBColor( {   0, 127,   0 } )
#define COLOR_ORANGE     GraMakeRGBColor( { 253, 120,  41 } )
#define COLOR_SILVER     GraMakeRGBColor( { 192, 192, 192 } )
#define COLOR_ICEGREEN   GraMakeRGBColor( { 222, 255, 189 } )

FUNCTION Main()

LOCAL GetList[0], GetOptions, oBrowse, oBrowseQuery, aQuery, oConfig

oConfig := DC_XbpPushButtonXPConfig():new()
oConfig:fgColorMouse    := COLOR_DARKGREEN
oConfig:bgColorMouse    := COLOR_ORANGE
oConfig:fgColor         := GRA_CLR_BLACK
oConfig:bgColor         := COLOR_SILVER
oConfig:fgColorSelected := GRA_CLR_GREEN
oConfig:bgColorSelected := COLOR_ICEGREEN
oConfig:fgColorClick    := COLOR_DARKGREEN
oConfig:bgColorClick    := COLOR_ICEGREEN
oConfig:gradientStep    := 10
oConfig:gradientReverse := .T.
oConfig:radius          := 12
oConfig:outline         := .T.
oConfig:font            := '12.Arial Bold'
oConfig:mouseOverFont   := '12.Arial Bold'
oConfig:mouseOverScale  := 1.1

DC_LoadRdds()

USE ..\..\DATA\CUSTOMER INDEX ..\..\DATA\CUSTOMER VIA 'FOXCDX'

@ 0,0 DCBROWSE oBrowse ALIAS 'CUSTOMER' SIZE 100,20 ;
      HEADLINES 2 RESIZE DCGUI_RESIZE_RESIZEONLY

DCBROWSECOL FIELD CUSTOMER->cust_nmbr HEADER 'Customer;Number' WIDTH 8 PARENT oBrowse
DCBROWSECOL FIELD CUSTOMER->bill_name HEADER 'Name' WIDTH 15 PARENT oBrowse
DCBROWSECOL FIELD CUSTOMER->bill_strt HEADER 'Street' WIDTH 15 PARENT oBrowse
DCBROWSECOL FIELD CUSTOMER->bill_city HEADER 'City' WIDTH 15 PARENT oBrowse
DCBROWSECOL FIELD CUSTOMER->bill_state HEADER 'State' WIDTH 2 PARENT oBrowse
DCBROWSECOL FIELD CUSTOMER->bill_zip HEADER 'Zip;Code' WIDTH 5 PARENT oBrowse
DCBROWSECOL FIELD CUSTOMER->phone HEADER 'Phone;Number' WIDTH 10 PARENT oBrowse

aQuery := {{"Number","CUST_NMBR",'@!','CUST_NMBR' }, ;
           {"Name","BILL_NAME","@!","BILL_NAME" }, ;
           {"City","BILL_CITY",'@!','BILL_CITY'}, ;
           {"State","BILL_STATE",'@!',nil}, ;
           {"Phone","PHONE",'@!','PHONE'}}

oBrowseQuery := DC_BrowseQuery():new(GetList)
oBrowseQuery:queryArray := aQuery
oBrowseQuery:alias := 'CUSTOMER'
oBrowseQuery:buttonConfig := oConfig
oBrowseQuery:bgColor := COLOR_ICEGREEN
oBrowseQuery:isVertical := .t.
oBrowseQuery:isFloating := .t.
oBrowseQuery:maxGetWidth := 20
oBrowseQuery:saySize := 7
oBrowseQuery:buildGets(@oBrowse)

@ 21, 0 DCPUSHBUTTONXP CAPTION "F&ind"  ;
        ACCELKEY xbeK_ALT_I SIZE 11,1.4 GROUP 'BUTTONS' ID 'BROWSEVIEW' ;
        CONFIG oConfig ;
        ACTION {||Eval(oBrowseQuery:startBlock)} ;
        WHEN {||!oBrowseQuery:isVisible()} ;
        TOOLTIP "Specify A Filtered View"

DCGETOPTIONS RESIZE

DCREAD GUI FIT TITLE 'Browse Query Test' SETAPPWINDOW OPTIONS GetOptions

RETURN nil

* -----------

PROC appsys ; RETURN

* ---------------

#DEFINE QUERY_SAY          1
#DEFINE QUERY_FIELD        2
#DEFINE QUERY_PICTURE      3
#DEFINE QUERY_INDEX        4
#DEFINE QUERY_ROW          5
#DEFINE QUERY_COL          6
#DEFINE QUERY_GET          7
#DEFINE QUERY_VAR          8

CLASS DC_BrowseQuery

EXPORTED:

VAR alias
VAR queryArray
VAR browseObject
VAR browseObjectBlock
VAR parentObject
VAR queryGetList
VAR mainGetList
VAR buttonConfig
VAR startBlock
VAR doneBlock
VAR bgColor
VAR staticRow
VAR staticCol
VAR staticWidth
VAR staticHeight
VAR isPixel
VAR isSQL
VAR saySize
VAR isFloating
VAR isVertical
VAR dialogWindow
VAR maxGetWidth

METHOD init, buildGets, applyQuery, testQuery, clearFields, isVisible, destroy

ENDCLASS

* ----------

METHOD DC_BrowseQuery:init( GetList )

::isSQL := .f.
::isFloating := .f.
::isVertical := .f.
::alias := Alias()
::mainGetList := GetList
::saySize := 15
::staticRow := 0
::staticCol := 0
::maxGetWidth := 100

RETURN self

* ----------

METHOD DC_BrowseQuery:buildGets( oBrowse, oStaticBrowse )

LOCAL i, nRow := .5, nCol := 0, dPopUp, nSaveRec := (::alias)->(RecNo()), lData := .F., ;
      nColOffset, nButtonCol, GetList, oStaticQuery, oStaticText, nWidth, GetOptions, ;
      nGetSize

nButtonCol := 1

IF ::isFloating
  GetList := {}
  oStaticBrowse := nil
ELSE
  GetList := ::mainGetList
  ::isVertical := .f.
ENDIF
::queryGetList := GetList

::browseObjectBlock := DC_GetAnchorCb(@oBrowse)

(::alias)->(dbGoto(0))

IF !::isFloating

  @  ::staticRow,  ::staticCol DCSTATIC TYPE XBPSTATIC_TYPE_RECESSEDBOX OBJECT oStaticQuery ;
                SIZE ::staticWidth, ::staticHeight PARENT oStaticBrowse ;
                EVAL {|o|IIF(!::isFloating,o:hide(),nil)}

  @   2,  2 DCSTATIC TYPE XBPSTATIC_TYPE_TEXT OBJECT oStaticText ;
                SIZE DCGUI_PARENTWIDTH-4, DCGUI_PARENTHEIGHT-4 ;
                PARENT oStaticQuery COLOR GRA_CLR_BLACK, ::bgColor ;
                PIXEL

ENDIF

nRow := .5

@ nRow, nButtonCol DCPUSHBUTTON CAPTION 'Find' SIZE 8,1 ;
         ACTION {||::applyQuery(@oBrowse)} PARENT oStaticText GROUP 'QUERYBUTTONS' ;
         WHEN {|| ::testQuery() .OR. lData } ;
         TOOLTIP "Find Records Matching The Selected Search Criteria" ;
         CONFIG ::buttonConfig

IF !::isVertical
  nRow++
ELSE
  nButtonCol += 10
ENDIF

@ nRow, nButtonCol DCPUSHBUTTON CAPTION 'Clear' SIZE 8,1 ;
         ACTION {||::clearFields(@oBrowse)} PARENT oStaticText GROUP 'QUERYBUTTONS' ;
         WHEN {|| ::testQuery() .OR. lData } ;
         TOOLTIP "Reset Search Criteria" ;
         CONFIG ::buttonConfig

IF !::isVertical
  nRow++
ELSE
  nButtonCol += 10
ENDIF

@ nRow, nButtonCol DCPUSHBUTTON CAPTION 'Done' SIZE 8,1 ;
         ACTION {||::clearFields(@oBrowse), ;
                   Eval(::doneBlock)} PARENT oStaticText ;
         TOOLTIP "Close Search Window" ;
         CONFIG ::buttonConfig

IF !::isVertical
  nColOffset := nButtonCol + 10
  nRow := .5
ELSE
  nColOffset := 0
  nRow += 2
ENDIF

nCol := 0
nWidth := 0

DEFAULT ::staticHeight := (Len(::queryArray) / 3 + 1)

FOR i := 1 TO Len(::queryArray)
  ASize(::queryArray[i], QUERY_VAR)
  ::queryArray[i,QUERY_ROW] := nRow
  ::queryArray[i,QUERY_COL] := nCol
  ::queryArray[i,QUERY_GET] := (::alias)->(&(::queryArray[i,QUERY_FIELD]))

  nGetSize := Len(DC_XtoC(::queryArray[i,QUERY_GET]))
  nGetSize := Min(nGetSize,::maxGetWidth)

  nWidth := Max( nWidth, nGetSize )

  nRow++
  IF !::isVertical .AND. nRow > ::staticHeight-1
    nRow := .5
    nCol += ::saySize + nWidth + 2
    nWidth := 0
  ENDIF
NEXT

FOR i := 1 TO Len(::queryArray)

  dPopup := nil
  nGetSize := nil
  IF Valtype(::queryArray[i,QUERY_GET]) == 'D'
    dPopup := {|d|DC_PopDate(d,,,,,,2)}
  ELSEIF Valtype(::queryArray[i,QUERY_GET]) == 'C'
    nGetSize := Len(::queryArray[i,QUERY_GET]) + 1
    nGetSize := Min(nGetSize,::maxGetWidth)
  ENDIF

  nRow := IIF( ValType(::queryArray[i,QUERY_ROW]) == 'N', ::queryArray[i,QUERY_ROW], nRow++ )
  nCol := IIF( ValType(::queryArray[i,QUERY_COL]) == 'N', ::queryArray[i,QUERY_COL], 0 ) + nColOffset

  @ nRow++, nCol DCSAY ::queryArray[i,QUERY_SAY] GET ::queryArray[i,QUERY_GET] ;
                 KEYBLOCK {|a,b,o| IIF( Len(AllTrim(o:Get:buffer)) <= 2, ;
                                       ( lData := !Empty(AllTrim(o:Get:buffer)), DC_GetRefresh(::queryGetList,,,,{'QUERYBUTTONS'})), nil )} ;
                 SAYSIZE ::saySize SAYRIGHTBOTTOM TABSTOP PARENT oStaticText POPUP dPopUp ;
                 GETCOLOR GRA_CLR_BLACK, GRA_CLR_WHITE ;
                 GETID 'QUERY_' + Alltrim(Str(i)) ;
                 GETEVAL _GetEvalBlock(::queryArray[i],ATail(::queryGetList)) ;
                 PICTURE ::queryArray[i,QUERY_PICTURE] ;
                 GETSIZE nGetSize

NEXT

DCEVAL {||::browseObject := oBrowse, ;
          ::parentObject := oStaticQuery}

IF ::isFloating

  ::startBlock := {||::dialogWindow:show(), ;
                    ::dialogWindow:toFront(), ;
                    SetAppFocus(DC_GetObject(::queryGetList,'QUERY_1')), ;
                    DC_GetRefresh(::mainGetList)}

  ::doneBlock := {||::dialogWindow:hide(), ;
                    DC_GetRefresh(::mainGetList)}

ELSE

  ::startBlock := {|| IIF( oStaticQuery:isVisible(), nil, ;
                         (oBrowse:setSize( {oBrowse:currentSize()[1],oBrowse:currentSize()[2]-oStaticQuery:currentSize()[2]}), ;
                         oBrowse:setPos( {oBrowse:currentPos()[1],oStaticQuery:currentSize()[2]}),;
                         oStaticQuery:show(), ;
                         DC_GetRefresh(::queryGetList), ;
                         SetAppFocus(DC_GetObject(::queryGetList,'QUERY_1'))))}

  ::doneBlock := {|| oStaticQuery:hide(), ;
                   oBrowse:setSize( oStaticBrowse:currentSize() ), ;
                   oBrowse:setPos( {0,0}), DC_GetRefresh(::queryGetList) }
ENDIF

(::alias)->(dbGoTo(nSaveRec))

IF ::isFloating
  DCGETOPTIONS COLOR ::bgColor HIDE ALWAYSONTOP
  DCREAD GUI FIT TITLE 'Browse Query' EXIT ;
      EVAL {|o|::dialogWindow := o, ;
               ::browseObject := oBrowse} ;
      OPTIONS GetOptions SAVE
ENDIF

RETURN nil

* ------------

STATIC FUNCTION _GetEvalBlock( aQueryItem, aGetListItem )

RETURN {|o|aQueryItem[QUERY_VAR] := aGetListItem[bGETLIST_VAR]}

* ------------

METHOD DC_BrowseQuery:isVisible()

IF ::isFloating
  RETURN ::dialogWindow:isVisible()
ENDIF

RETURN ::parentObject:isVisible()

* ------------

METHOD DC_BrowseQuery:destroy()

IF Valtype(::dialogWindow) == 'O'
  ::dialogWindow:destroy()
ENDIF

RETURN self

* ------------

METHOD DC_BrowseQuery:applyQuery( oBrowse )

LOCAL i, oDlg, aRecords[0], lAdd, nCnt, nPct, nLastRec, oProgress, GetList[0], GetOptions, ;
      xScope, lCancelled := .f.

IF !::testQuery()
  DC_WinAlert('No Search Criteria Entered!!')
  RETURN nil
ENDIF

nLastRec := (::alias)->(LastRec())
nPct := Round(nLastRec/100,0)
nCnt := 0

@ 0,0 DCPROGRESS oProgress RADIUS 20 DYNAMIC OUTLINE COLOR GRA_CLR_CYAN, GRA_CLR_WHITE SIZE 50,1 PERCENT
@ DCGUI_ROW, DCGUI_COL + 10 DCPUSHBUTTON CAPTION 'Cancel' SIZE 10,1 CONFIG ::buttonConfig ;
   ACTION {||lCancelled := .t.}

DCGETOPTIONS COLOR GRA_CLR_WHITE
DCREAD GUI FIT TITLE 'Searching...' EXIT PARENT @oDlg OPTIONS GetOptions

DC_PointerWait()
(::alias)->(DC_ClrScope())
(::alias)->(DC_SetScopeArray(nil))

FOR i := 1 TO Len(::queryArray)
  IF !Empty(::queryArray[i,QUERY_GET]) .AND. Upper(DC_XtoC(::queryArray[i,QUERY_INDEX])) == (::alias)->(OrdSetFocus())
    IF Valtype(::queryArray[i,QUERY_GET]) = 'C'
      xScope := Trim(::queryArray[i,QUERY_GET])
    ELSEIF Valtype(::queryArray[i,QUERY_GET]) = 'D'
      xScope := Dtos(::queryArray[i,QUERY_GET])
    ELSE
      xScope := ::queryArray[i,QUERY_GET]
    ENDIF
    (::alias)->(DC_SetScope(0,xScope))
    (::alias)->(DC_SetScope(1,xScope))
    EXIT
  ENDIF
NEXT

(::alias)->(DC_dbGoTop())

DO WHILE !(::alias)->(Eof()) .AND. !lCancelled
  IF nCnt % 100 == 0
    DC_CompleteEvents()
  ENDIF
  IF ++nCnt%nPct == 0
    DC_GetProgress( oProgress, nCnt, nLastRec )
  ENDIF
  lAdd := .f.
  FOR i := 1 TO Len(::queryArray)
    IF !Empty(::queryArray[i,QUERY_GET])
      IF Valtype(::queryArray[i,QUERY_GET]) = 'C' .AND. Upper((::alias)->(&(::queryArray[i,QUERY_FIELD]))) = Upper(Trim(::queryArray[i,QUERY_GET]))
        lAdd := .t.
      ELSEIF Valtype(::queryArray[i,QUERY_GET]) = 'D' .AND. DTOS((::alias)->(&(::queryArray[i,QUERY_FIELD]))) = DTOS(::queryArray[i,QUERY_GET])
        lAdd := .t.
      ELSEIF (::alias)->(&(::queryArray[i,QUERY_FIELD])) == ::queryArray[i,QUERY_GET]
        lAdd := .t.
      ELSE
        lAdd := .f.
        EXIT
      ENDIF
    ENDIF
  NEXT
  IF lAdd
    AAdd(aRecords,(::alias)->(RecNo()))
  ENDIF
  (::alias)->(DC_DbSkip())
ENDDO
oDlg:destroy()

IF Empty(aRecords)
  aRecords := {-1 }
ENDIF

(::alias)->(DC_ClrScope())
IF lCancelled
  (::alias)->(DC_SetScopeArray(nil))
ELSE
  (::alias)->(DC_SetScopeArray(aRecords))
ENDIF
(::alias)->(DC_DbGoTop())

oBrowse:refreshAll()
DC_PointerArrow()

IF aRecords[1] = -1 .AND. !lCancelled
  DCMSGBOX "No Matching Records Were Found" TITLE "Data Alert" TIMEOUT 3
ENDIF

RETURN nil

* -----------

METHOD DC_BrowseQuery:testQuery()

LOCAL i, lStatus := .f.

FOR i := 1 TO Len(::queryArray)
  IF !Empty(::queryArray[i,QUERY_GET])
    lStatus := .t.
    EXIT
  ENDIF
NEXT

RETURN lStatus

* -----------

METHOD DC_BrowseQuery:clearFields( oBrowse )

LOCAL i

(::alias)->(dbGoTo(0))

FOR i := 1 TO Len(::queryArray)
  ::queryArray[i,QUERY_GET] := (::alias)->(&(::queryArray[i,QUERY_FIELD]))
NEXT

(::alias)->(DC_SetScopeArray(nil))
(::alias)->(DC_DbGoTop())

DC_GetRefresh(::queryGetList)
DC_GetRefresh(::mainGetList)
// oBrowse:refreshAll()

RETURN nil





